@@ -2,9 +2,9 @@ import os, sys, cgi, re
 import gobject,gtk, gnome, gnome.ui, gnomevfs
 import deskbar, deskbar.Handler, deskbar.Utils, deskbar.Match
 from gettext import gettext as _
-from os.path import exists
+from os.path import exists, dirname
 from deskbar.defs import VERSION
-from deskbar.Utils import is_program_in_path, spawn_async
+from deskbar.Utils import is_program_in_path, spawn_async, url_show, url_show_file
 
 MAX_RESULTS = 20 # per handler
 
@@ -103,7 +103,7 @@ TYPES = {
 		},
 	"File" 		: {
 		"name"	: ("beagle:ExactFilename",), 
-		"action": lambda d: gnome.url_show(d["uri"]),
+		"action": lambda d: url_show_file("file://"+d["uri"]),
 		"icon"	: "stock_new",
 		#translators: This is a file.
 		"description": _("Open %s") % "<b>%(name)s</b>",
@@ -112,7 +112,7 @@ TYPES = {
 		},
 	"FeedItem"	: {
 		"name"	: ("dc:title",),
-		"action": lambda d: gnome.url_show(d["identifier"]),
+		"action": lambda d: url_show(d["identifier"]),
 		"icon"	: "stock_news",
 		"description": (_("News from %s") % "<i>%(publisher)s</i>" ) + "\n<b>%(name)s</b>",
 		"snippet": True,
@@ -145,7 +145,7 @@ TYPES = {
 		},
 	"WebHistory": {
 		"name"	: ("dc:title",), # FIX-BEAGLE bug #330053, dc:title returns as None even though it _is_ set
-		"action": lambda d: gnome.url_show(d["uri"]),
+		"action": lambda d: url_show_file(d["uri"]),
 		"icon"	: "stock_bookmark",
 		"description": (_("Open History Item %s") % "<i>%(name)s</i>") + "\n%(escaped_uri)s",
 		"category": "web",
@@ -220,7 +220,6 @@ class BeagleLiveMatch (deskbar.Match.Match):
 		
 		action = TYPES[self.result["type"]]["action"]
 		if callable(action):
-			print "BeagleLive url_show()", self.result
 			action(self.result)
 		else:
 			# Retrieve the associated action
